/*
 * $Id: cmdline.c,v 1.6 2000/06/01 09:38:39 athena Exp $
 */

/*
 * Copyright (c) 2000 Massachusetts Institute of Technology
 * Copyright (c) 2000 Matteo Frigo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/***********************************************************\
 * Command line parsing
\***********************************************************/

#include <cilk.h>
#include <cilk-internal.h>
#include <stdio.h>
#include <stdlib.h>

enum {
     NONE, NPROC, STATS, NO_STATS, HELP, STACK, YIELD, NO_YIELD,
     POSIX_LOCKS, MEMORY_LOCKS,
     INFOFILE, DUMP_CORE, NO_DUMP_CORE, ALLOC_BATCH, VERSION_, END_OPTIONS
};

static struct options {
     char *string;
     int option;
     char *help;
} optarray[] = {
     {
	  "", END_OPTIONS, "-- : end of option parsing"
     },
     {
	  "nproc", NPROC, "--nproc <n> : set number of processors"
     },
     {
	  "stack", STACK, "--stack <n> : set stack depth"
     },
     {
	  "stats", STATS, "--stats <n> : set statistics level"
     },
     {
	  "no-stats", NO_STATS, "--no-stats : disable statistics printing"
     },
     {
	  "alloc-batch", ALLOC_BATCH,
	  "--alloc-batch <n> : set batch length for memory allocator"
     },
     {
	  "yield", YIELD, "--yield : yield time slice when steal fails"
     },
     {
	  "no-yield", NO_YIELD, "--no-yield : don't yield time slice"
     },
     {
	  "with-posix-locks", POSIX_LOCKS, "--with-posix-locks : use posix locks"
     },
     {
	  "with-memory-locks", MEMORY_LOCKS, "--with-memory-locks : use memory locks"
     },
     {
	  "dump-core", DUMP_CORE,
	  "--dump-core : dump core on fatal cilk errors"
     },
     {
	  "no-dump-core", NO_DUMP_CORE, 
	  "--no-dump-core : do not dump core on fatal cilk errors"
     },
     {
	  "version", VERSION_, 
	  "--version : print information about the cilk runtime system"
     },
     {
	  "help", HELP, "--help : print this message"
     },
     {
	  (char *) 0, NONE, ""
     }
};

static void print_help(void)
{
     struct options *p;

     fprintf(stderr, "Cilk options:\n");

     for (p = optarray; p->string; ++p)
	  if (p->help)
	       fprintf(stderr, "     %s\n", p->help);

     fprintf(stderr, "\n");
}

static void print_version(void)
{
     int debug = 0, stats = 0, timing = 0;
     WHEN_CILK_DEBUG(debug = 1);
     WHEN_CILK_STATS(stats = 1);
     WHEN_CILK_TIMING(timing = 1);
     fprintf(stderr, "Cilk version " VERSION "\n");
     fprintf(stderr, "compilation options: ");
     if (debug) fprintf(stderr, "CILK_DEBUG ");
     if (stats) fprintf(stderr, "CILK_STATS ");
     if (timing) fprintf(stderr, "CILK_TIMING ");
     if (!(debug | stats | timing))
	  fprintf(stderr, "none");
     fprintf(stderr, "\n");
}

/* look for a given string in the option table */
static struct options *parse_option(char *s)
{
     struct options *p;

     for (p = optarray; p->string; ++p)
	  if (strcmp(s, p->string) == 0)
	       break;

     return p;
}

#define CHECK(cond, complaint) \
     if (!(cond)) { fprintf(stderr, "Bad option argument for -%s: %s\n", p->string, complaint); return 1; }

int Cilk_parse_command_line(Cilk_options *options, int *argc, char *argv[])
{
     int i, j;
     struct options *p;

     /* default options */
     *options = CILK_DEFAULT_OPTIONS;

     for (i = 1; i < *argc; ++i) {
	  if (argv[i][0] != '-')
	       goto arch_specific;

	  if (argv[i][1] == '-')
	       p = parse_option(argv[i] + 2);
	  else {
	       /* accept deprecated old-style form -X instead of --X */
	       p = parse_option(argv[i] + 1);
	  }

	  switch (p->option) {
	      case NPROC:
		   ++i;
		   CHECK(i < *argc,
			 "argument missing");
		   options->nproc = atoi(argv[i]);
		   if (options->nproc == 0)
			options->nproc = Cilk_partition_size();
		   CHECK(options->nproc > 0,
			 "non-positive number of processors");
		   break;
	      case STACK:
		   ++i;
		   CHECK(i < *argc,
			 "argument missing");
		   options->stackdepth = atoi(argv[i]);
		   CHECK(options->stackdepth > 0,
			 "non-positive stack depth");
		   break;
	      case STATS:
		   ++i;
		   CHECK(i < *argc,
			 "argument missing");
		   options->statlevel = atoi(argv[i]);
		   break;
	      case NO_STATS:
		   options->statlevel = 0;
		   break;
	      case YIELD:
		   options->yieldslice = 1;
		   break;
	      case NO_YIELD:
		   options->yieldslice = 0;
		   break;
	      case POSIX_LOCKS:
		   options->memory_locks = 0;
		   break;
	      case MEMORY_LOCKS:
		   options->memory_locks = 1;
		   break;
	      case DUMP_CORE:
		   options->dump_core = 1;
		   break;
	      case NO_DUMP_CORE:
		   options->dump_core = 0;
		   break;
	      case HELP:
		   print_help();
		   return 1;
		   break;
	      case VERSION_:
		   print_version();
		   return 1;
		   break;
	      case ALLOC_BATCH:
		   ++i;
		   CHECK(i < *argc,
			 "argument missing");
		   options->alloc_batch_size = atoi(argv[i]);
		   if (options->alloc_batch_size < 8)
			options->alloc_batch_size = 8;
		   break;
	      case INFOFILE:
		   ++i;
		   CHECK(i < *argc,
			 "argument missing");
		   options->infofile_name = argv[i];
		   break;
	      case END_OPTIONS:
		   ++i;
		   goto stop;
	      default:
		   goto arch_specific;
	  }
     }

     /* let architecture-specific code get a crack at the command line */
   arch_specific:
     i = Cilk_arch_specific_options(i, *argc, argv);

     /* remove Cilk command-line arguments from argv */
   stop:
     for (j = i; j <= *argc; j++)
	  argv[j - (i - 1)] = argv[j];
     *argc -= i - 1;

     return 0;
}
